/****************************************************************************
   Program:     $Id: messages.c,v 1.19 2009/01/13 19:00:15 rbeverly Exp $
   Author:      Rob Beverly <rbeverly at csail.mit.edu>
   Date:        $Date: 2009/01/13 19:00:15 $
   Description: Spoofer control channel message routines
****************************************************************************/
#include "spoof.h"

/******************** message printing/debug ********************/

char *type(uint8_t typ) {
    static char msg_type_name[SMALLBUF];
    switch(typ){
        case DEFAULT:
            strncpy(msg_type_name, "DEFAULT", sizeof(msg_type_name)); 
            break;
        case HELLO:
            strncpy(msg_type_name, "HELLO", sizeof(msg_type_name)); 
            break;
        case PROBE_REQ:
            strncpy(msg_type_name, "PROBE_REQ", sizeof(msg_type_name)); 
            break;
        case REPORT:
            strncpy(msg_type_name, "REPORT", sizeof(msg_type_name)); 
            break;
        case TRACE:
            strncpy(msg_type_name, "TRACE", sizeof(msg_type_name)); 
            break;
        case ERR:
            strncpy(msg_type_name, "ERR", sizeof(msg_type_name)); 
            break;
        case DONE:
            strncpy(msg_type_name, "DONE", sizeof(msg_type_name)); 
            break;
        default:
            strncpy(msg_type_name, "INVALID", sizeof(msg_type_name)); 
            break;
    }
    return (msg_type_name);
}

void printAddressList(struct in_addr *iplist, int num) {
    int i;
    for (i=0;i<num;i++) {
        if (&iplist[i] == NULL) return;
        printf("\tIP: %s\n", inet_ntoa(iplist[i]));
    }
}

void printTest(test_t *test, int detail) {
    static char buf[SMALLBUF];
    struct in_addr a;

    snprintf(buf, SEQNOSIZE, "%s", test->seqno);
    a.s_addr = test->src_addr;
    printf(">> Probe Src: %s ", inet_ntoa(a));
    a.s_addr = test->dst_addr;
    printf("Dst: %s ", inet_ntoa(a));
    if (detail) {
        printf("Seq: %s HMAC: ", buf);
        binDump(test->hmac, HMACSIZE);
    }
}

void printTestList(unsigned char *msg, int verbose) {
    addr_list_t *list = (addr_list_t *) msg; 
    test_t *test;
    int i;

    if (ntohs(list->size) > 100) {
        warn("bogus test list.");
        return;
    }
    if (verbose) {
        test = (test_t *) (msg + sizeof(addr_list_t));
        for (i=0;i<ntohs(list->size);i++) {
            printTest(test, 1);
            test++;
        }
    }
}

char *printCharStr(unsigned char *str, int len) {
    static char buf[BIGBUF];
    memset(buf, 0, sizeof(buf));
    snprintf(buf, len+1, "%s", str);
    return (buf);
}

void printHelloMsg(unsigned char *msg, int verbose) {
    hello_msg_t *hello = (hello_msg_t *) msg; 
    unsigned char *os = msg + sizeof(hello_msg_t);
    struct in_addr addr;
    addr.s_addr = hello->ip;
    printf(">> HELLO: [Client: %s][Dist: %d][OSlen: %d]",
            inet_ntoa(addr), hello->dist, hello->oslen);
    if (hello->oslen >0) 
        printf("[OS: %s]", printCharStr(os, hello->oslen));
    printf("[SID: %lu]\n", (unsigned long) ntohl(hello->sessionid));
}

void printReportMsg(unsigned char *msg) {
    report_msg_t *report = (report_msg_t *) msg;
    unsigned char *seq = msg + sizeof(report_msg_t);
    struct in_addr addr;
    printf("Report Message:\n");
    addr.s_addr = report->src_ip;
    printf("\tSrc IP: %s\n", inet_ntoa(addr));
    addr.s_addr = report->dst_ip;
    printf("\tDst IP: %s\n", inet_ntoa(addr));
    if (report->seqlen > 0) {
        printf("\tSeq(%d): ", report->seqlen);
        printf("%s\n", printCharStr(seq, report->seqlen));
    }
}

void printMsg(unsigned char *msg, int verbose) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    unsigned char *payload;
    int len = ntohs(hdr->len);
    if (!verbose) {
        printf(">> %s Msg v%d: [Flag: %d][Ack: %d][Len: %d]\n",
            type(hdr->type), hdr->ver, hdr->flag, hdr->ack, len);
    } else {
        printf("Message Header:\n");
        printf("\tVer:  %d\n", hdr->ver);
        printf("\tType: %s\n", type(hdr->type));
        printf("\tFlag: %d\n", hdr->flag);
        printf("\tAck:  %d\n", hdr->ack);
        printf("\tLen:  %d\n", len);
    }
    if (len == 0) return;
    payload = msg + sizeof(spoof_msg_t);
    switch(hdr->type){
        case HELLO:
            printHelloMsg(payload, verbose);
            break;
        case PROBE_REQ:
            printTestList(payload, verbose);
            break;
        case REPORT:
            printReportMsg(payload);
            break;
        case TRACE:
            printf("%s\n", printCharStr(payload, len));
            break;
        case ERR:
            printCharStr(payload, len);
            break;
        case DONE:
            if (verbose) printf("%s\n", printCharStr(payload, len));
            break;
        default:
            printf("*** Error, invalid message payload\n");
    }
}


/******************** message kraftwerk ********************/

int readMsg(int sock, spoof_msg_t **message) {
    unsigned char *buf = (unsigned char *) *message; 
    spoof_msg_t *msg = *message;
    
    int n = Readn(sock, buf, sizeof(spoof_msg_t));
    msg = (spoof_msg_t *) buf;
    if (ntohs(msg->len) > MAXMSGSIZE)
        return 0;
    if (ntohs(msg->len) > 0) {
        if ((n+= Readn(sock, buf + sizeof(spoof_msg_t), ntohs(msg->len))) <= 0)
            return 0;   /* connection closed by other end */
#if DEBUG 
        printMsg(buf, VERBOSE);
#endif
    }
    return(n);
}

void defaultMsg(spoof_msg_t *msg) {
    if (msg) {
        msg->ver= VERSION;
        msg->type = DEFAULT;
        msg->flag = 0;
        msg->ack = 0;
        msg->len = 0;
        msg->unused = 0;
    }
}

int errorRequest(unsigned char *msg) {
    int msglen = sizeof(spoof_msg_t);
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    
    defaultMsg(hdr);
    hdr->type = ERR;
    return(msglen);
}

int helloRequest(unsigned char *msg) {
    int oslen = strlen(OS);
    int msglen = sizeof(spoof_msg_t) + sizeof(hello_msg_t) + oslen;
    /* TODO: assert(sizeof(*msg) >= msglen) */
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    hello_msg_t *hello = (hello_msg_t *) (msg + sizeof(spoof_msg_t));
    unsigned char *os = msg + sizeof(spoof_msg_t) + sizeof(hello_msg_t);
    
    defaultMsg(hdr);
    hdr->type = HELLO;
    hdr->len = htons(sizeof(hello_msg_t) + oslen);
    hello->ip = 0;
    hello->dist = 0;
    hello->unused = 0;
    hello->oslen = oslen;
    memcpy(os, OS, oslen);
    hello->sessionid = 0;
    return(msglen);
}

int reportRequest(unsigned char *msg, unsigned char *seqno, 
        unsigned long src_addr, unsigned long dst_addr) {
    int msglen = sizeof(spoof_msg_t) + sizeof(report_msg_t) + SEQNOSIZE;
    /* TODO: assert(sizeof(*msg) >= msglen) */
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    report_msg_t *report = (report_msg_t *) (msg + sizeof(spoof_msg_t));
    unsigned char *seq = msg + sizeof(spoof_msg_t) + sizeof(report_msg_t);
    
    defaultMsg(hdr);
    hdr->type = REPORT;
    hdr->len = htons(sizeof(report_msg_t) + SEQNOSIZE);
    report->src_ip = src_addr; 
    report->dst_ip = dst_addr; 
    report->seqlen = SEQNOSIZE;
    /* XXX: this is a legacy hack.  Rob needs to fix */
    if (strstr((char *)seqno, "SENDTOFAIL") == NULL) {
        report->ack = 1;
    }
    memcpy(seq, seqno, SEQNOSIZE);
    return(msglen);
}

int doneRequest(unsigned char *msg) {
    int msglen = sizeof(spoof_msg_t);
    spoof_msg_t *hdr = (spoof_msg_t *) msg;

    defaultMsg(hdr);
    hdr->type = DONE;
    return(msglen);
}

int probeRequest(unsigned char *msg) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    defaultMsg(hdr);
    hdr->type = PROBE_REQ;
    return (sizeof(spoof_msg_t));
}

int readProbeList(unsigned char *buf, test_t **testlistptr) {
    test_t *testlist;
    addr_list_t *addr = (addr_list_t *) (buf + sizeof(spoof_msg_t));
    int numAddr = ntohs(addr->size);
    int i;

    *testlistptr = calloc(numAddr, sizeof(test_t));
    testlist = *testlistptr;
    for (i=0;i<numAddr;i++) {
        memcpy(testlist + i, 
            buf + sizeof(spoof_msg_t) + sizeof(addr_list_t) + (sizeof(test_t)*i), 
            sizeof(test_t));
    }
    return(numAddr);
}
