/****************************************************************************
   Program:     $Id: spoofer.c,v 1.46 2009/01/13 21:09:37 rbeverly Exp $
   Author:      Rob Beverly <rbeverly at csail.mit.edu>
   Date:        $Date: 2009/01/13 21:09:37 $
   Description: Spoofer - generate spoofed packets
****************************************************************************/
#include "spoof.h"

void procErr(int sock) {
    unsigned char buf[BIGBUF];
    spoof_msg_t *msg = (spoof_msg_t *) buf;
    int n;

    n = errorRequest(buf);
    Writen(sock, buf, n);
    n = readMsg(sock, &msg);
    if ( (msg->type == ERR) && (msg->ack == 1) && (msg->flag == 1) ) 
        warn("%s", &buf[sizeof(spoof_msg_t)]);
    else
        warn("Error, but unable to ask server why.");
}


void sendProbes(int sock, int num, test_t *test_ptr, 
               struct in_addr clientip, unsigned long sessionid, int spoof) {
    int i,j;
    unsigned char buf[BIGBUF];
    unsigned char pkt[BIGBUF];
    struct sockaddr_in src, dst;
    int packetlen = 0;
    int ret;

    src.sin_family = AF_INET;
    src.sin_port = htons(SRC_PORT);
    dst.sin_family = AF_INET;
    dst.sin_port = htons(SERV_PORT);

#if DEBUG==0
    if (spoof == TRUE) {
        printf(">> Sourcing spoofed probes (please wait)");
    } else {
        printf(">> Sourcing non-spoofed validation probes");
    }
#endif

    for (i=0; i<num; i++, test_ptr++) {
        fflush(stdout);
        dst.sin_addr.s_addr = test_ptr->dst_addr;

        /* craft probe packets */
        if ( (spoof == TRUE) && (test_ptr->src_addr != clientip.s_addr) ) {
            src.sin_addr.s_addr = test_ptr->src_addr;
            craftReport(buf, sessionid, test_ptr);
            packetlen = craftPacket(pkt, buf, PROBESIZE, &src, &dst, TTL_OUT);
#if DEBUG
            printf(">> Source %d spoofed packets (%s->", DUPLICATES,  inet_ntoa(src.sin_addr));
            printf("%s)...\n", inet_ntoa(dst.sin_addr));
#else
	    printf("%d..", i);
#endif
        } else if ( (spoof == FALSE) && (test_ptr->src_addr == clientip.s_addr) ) {
            craftReport(pkt, sessionid, test_ptr);
#if DEBUG
            printf(">> Source %d non-spoofed packets to %s...\n", DUPLICATES, inet_ntoa(dst.sin_addr));
#else
	    printf("..");
#endif
        } else {
            continue;
        }

        /* send probe packets */
        for (j = 0; j < DUPLICATES; j++) {
            if (spoof) {
                ret = socketSendSpoof(sock, pkt, packetlen, &dst, sizeof(dst));
                if (ret == WINRAWSOCK_ERROR) {
                    /* after winrawsock err, socketSendSpoof uses raw 
                    ethernet frames.  try again. */
                    j--;
                } else if (ret == SOCKET_ERROR) {
                    strncpy((char *)test_ptr->seqno, "SENDTOFAIL", strlen("SENDTOFAIL")+1);
                    /* don't try to send duplicates on error */
                    j += DUPLICATES;
                } else if (ret == SENDTO_ERROR) {
                    strncpy((char *)test_ptr->seqno, "SENDTOFAIL", strlen("SENDTOFAIL")+1);
                }
            } else {
                socketSend(sock, pkt, PROBESIZE, &dst, sizeof(dst));
            }
            spoofSleep();
        }
    }
#if DEBUG==0
    printf("\n");
#endif
}


void sendTraceFilter(int sock, int pathdist, short sessionid) {
    struct sockaddr_in src, dst;
    unsigned char *icmpkey;
    int packetlen;
    int i,j;
    unsigned char pkt[BIGBUF];

    src.sin_family = AF_INET;
    dst.sin_family = AF_INET;
    dst.sin_port = htons(SERV_PORT);
    src.sin_addr.s_addr = inet_addr(SERVER);
    dst.sin_addr.s_addr = inet_addr(VALIDDST);

    /* we encode the session ID into the UDP source port */
    src.sin_port = sessionid;
    printf(">> Performing reverse traceroute [TTL: ");
    for (i = 1; i <= pathdist; i++) {
        fflush(stdout);
        /* the TTL of the packet will be encoded as the size of the spoofed
         * UDP datagram.  TTL = UDPlength - 8 */
        icmpkey = calloc(i, sizeof(char));
        packetlen = craftPacket(pkt, icmpkey, i, &src, &dst, i);
	printf("%d..", i);
        for (j = 0; j < DUPLICATES; j++) {
            socketSendSpoof(sock, pkt, packetlen, &dst, sizeof(dst));
            spoofSleep();
        }
        free(icmpkey);
    }
    printf("]\n");
}


void collectResults(int sock, int num, test_t *test_ptr, struct in_addr clientip) {
    unsigned char buf[BIGBUF];
    int i, n;
    spoof_msg_t *msg = (spoof_msg_t *) buf;
    
    printf("\n>> Collecting Results:\n");

    for (i = 0; i < num; i++) {
        fflush(stdout);
        if (test_ptr->src_addr != clientip.s_addr) {
            n = reportRequest(buf, test_ptr->seqno, test_ptr->src_addr, test_ptr->dst_addr);
            Writen(sock, buf, n);
            n = readMsg(sock, &msg);
            if (msg->type == REPORT) {
                printTest(test_ptr, VERBOSE);
                printf("Received: %d\n",  msg->ack);
            }
        }
        test_ptr++;
    }
}


int main(int argc, char **argv) {
    spoof_msg_t *msg;
    hello_msg_t *hello;
    struct sockaddr_in dst;
    struct in_addr clientip;
    test_t *tests = NULL;
    unsigned char buf[BIGBUF];
    char tracebuf[BIGBUF];
    char sessionkey[SMALLBUF];
    unsigned long sessionid = 0;
    unsigned long pathdist = 0;
    int spoofsock = 0, ctrlsock = 0, udpsock = 0;
    int numProbes = 0;
    int n;

    /* Init */
    banner();
    checkroot();
    srand(time(NULL));

#ifdef _WIN32
    win_init();
    if (findInterfaces() > 1) {
        printf("** Warning: found more than one numbered network interface:\n");
        selectInterface();
    }
#endif

    /* Establish TCP control connection */
    dst.sin_family = AF_INET;
    dst.sin_addr.s_addr = inet_addr(SERVER);
    dst.sin_port = htons(REPORT_PORT);
    ctrlsock = newSocket(SOCK_STREAM, IPPROTO_TCP);
    connectSock(ctrlsock, &dst, sizeof(dst));

    /* Create non-spoofing UDP socket for sanity check */
    udpsock = newSocket(SOCK_DGRAM, IPPROTO_UDP);

    /* Force TTL for non-spoofed UDP packets (measure hop distance) */
    optSocketTTL(udpsock, TTL_OUT);

    /* Create UDP spoofing socket */
    spoofsock = newSocket(SOCK_RAW, IPPROTO_UDP);
    optSocketHDRINCL(spoofsock);

    /* beginning of buf is always a spoof_msg_t */
    msg = (spoof_msg_t *) buf;

    /* Say Hello. */
    printf(">> Negotiating test scenario with server...\n");
    n = helloRequest(buf);
    Writen(ctrlsock, buf, n);
    n = readMsg(ctrlsock, &msg);
    hello = (hello_msg_t *) (buf + sizeof(spoof_msg_t));
    
    /* Ensure sane server response */
    if (msg->type != HELLO)
        fatal("Unable to communicate with server!\n");    
    if (msg->ver != VERSION) {
        procErr(ctrlsock);
        snprintf(sessionkey, 80, "BADVERSION");
        goto finalize;
    }

    /* Play nicely. */
    if (msg->ack == 0) {
        procErr(ctrlsock);
        fatal(">> This client unable to communicate with server.  Bye.\n");
    }

    /* Return hello provides my IP, session ID */
    clientip.s_addr = hello->ip;
    sessionid = ntohl(hello->sessionid);
    if (VERBOSE) {
        printf("Session:\n");
        printf("\tId: %lu\n", sessionid);
        printf("\tIP: %s (RFC1918? %d)\n", inet_ntoa(clientip), checkRFC1918(&clientip));
    }

    /* Grab test list */
    n = probeRequest(buf);
    Writen(ctrlsock, buf, n);
    n = readMsg(ctrlsock, &msg);
    if ( (msg->type != PROBE_REQ) || (msg->ack != TRUE) || (msg->flag != TRUE)) {
        procErr(ctrlsock);
        fatal("Couldn't grab test scenario from server.");
    } 
    numProbes = readProbeList(buf, &tests);
    if (numProbes <= 0) {
        procErr(ctrlsock);
        fatal("Couldn't grab test scenario from server.");
    }
    if (VERBOSE)
        printTestList(buf + sizeof(spoof_msg_t), TRUE);

    /* Ask for confirmation before proceeding */
    printf(">> Test scenario with %d probes\n>>\n>> Ready to test.\n", 
        numProbes);
    confirm(); 

    /* Make certain non-spoofed packets can reach the server */
    sendProbes(udpsock, numProbes, tests, clientip, sessionid, FALSE);

    /* Hello again after sending non-spoofed probes which gives path distance */
    n = helloRequest(buf);
    Writen(ctrlsock, buf, n);
    n = readMsg(ctrlsock, &msg);

    /* Sanity check path distance */
    pathdist = hello->dist;
    if (pathdist > TTL_OUT) {
        if (pathdist == 0xff)
            printf(">> ** no path distance available!\n");
        else
            printf(">> ** bogus distance response");
        pathdist = 0;
    }

#ifdef _WIN32
    /* wait to register ethernet until we've already communicated with
       the server -- this ensures there is an ARP entry waiting for us */
    registerEther();
#endif

    /* Send spoofed packets */
    sendProbes(spoofsock, numProbes, tests, clientip, 0, TRUE);

    /* Send TTL-limited reverse traceroute spoofed packets */
    sendTraceFilter(spoofsock, pathdist, sessionid);

    /* We send this final round of non-spoofed UDP packets to determine 
       if the TTL (hop-count) changed during the test */
    sendProbes(udpsock, numProbes, tests, clientip, sessionid, FALSE);

    /* Confer with server to get results */
    collectResults(ctrlsock, numProbes, tests, clientip);

    /* run and send trace */
    if (runtrace(SERVER, tracebuf) >= 0) {
        defaultMsg(msg);
        msg->type = TRACE;
        msg->len = htons(strlen(tracebuf));
        memcpy(buf + sizeof(spoof_msg_t), tracebuf, strlen(tracebuf));        
        Writen(ctrlsock, buf, sizeof(spoof_msg_t) + strlen(tracebuf));
        n = readMsg(ctrlsock, &msg);
    }

    printf(">> Finishing test...\n");
    n = doneRequest(buf);
    Writen(ctrlsock, buf, n);
    n = readMsg(ctrlsock, &msg);
    snprintf(sessionkey, ntohs(msg->len) + 1, "%s", buf + sizeof(spoof_msg_t));
    
finalize:
    close(udpsock);
    close(spoofsock);
    close(ctrlsock);
    if (tests)
    	free(tests);

    snprintf((char *)buf, SMALLBUF*2, "http://%s/report.php?sessionkey=%s", REPORT_HOST, sessionkey);
    printf("\nTest Complete.\n");
    printf("Your test results:\n\t%s\n\n", buf);
#ifdef _WIN32
    deregisterEther();
    /* spawn browser window with the results */
    ShellExecute(NULL, "open", buf, NULL, NULL, SW_SHOWNORMAL);
    winpause("Press Enter to Exit.");
#endif
    exit(0);
}
