/****************************************************************************
   Program:     $Id: trace.c,v 1.13 2009/01/13 00:39:33 rbeverly Exp $
   Author:      Rob Beverly <rbeverly at csail.mit.edu>
   Date:        $Date: 2009/01/13 00:39:33 $
   Description: Traceroute wrapper
****************************************************************************/
#include "spoof.h"

#ifndef _WIN32
#include <signal.h>

#define TRWAIT 6	// six second timeout on select() reading traceroute pipe
#define MAXWAIT 30  // maximum wait of 30 seconds 
#define NUM_TR_EXEC 5

pid_t pid;

/**
 * findtr - Locate traceroute binary on system
 *
 * @param[out]  traceroute      path to traceroute binary
 */
int findtr(char *traceroute) {
    char tr[NUM_TR_EXEC][SMALLBUF] = { 	
        "/usr/local/sbin/traceroute",
        "/usr/sbin/traceroute",
        "/sbin/traceroute",
        "/usr/bin/traceroute",
        "/usr/sbin/tracepath",
        };	
    FILE *fp;
    int i;

    for (i=0;i<NUM_TR_EXEC;i++) {
        fp = fopen(tr[i], "r");
        if (fp != NULL) {
            fclose(fp);
            snprintf(traceroute, BUFSIZE, "%s", tr[i]);
            return(1);
        }
    }
    /* fall through - couldn't find a traceroute on the system */
    if (DEBUG) printf("** No traceroute found on system\n");
        return(-1);
}

void catch() {
    // Parent process caught the ALRM, terminal (long running) child
    kill(pid, SIGKILL);
}

/**
 * runtrace - perform a traceroute to a destination 
 *
 * @param[in]   dest        destination
 * @param[out]  result      traceroute output
 *
 * Performs a traceroute to a destination.  Finds the
 * traceroute binary, starts time-limited trace.  Will
 * alarm out after TRWAIT seconds
 */
int runtrace(char *dest, char *result) {
    struct timeval timeout;
    char tracecmd[BUFSIZE];
    int bytes_read = 0;
    int pipefd[2];
    fd_set fds;

    if (findtr(tracecmd) < 0) 
       return(-1);

#if DEBUG
    printf(">> Running Trace (please wait): %s to %s\n", tracecmd, dest);
#else
    printf(">> Running Trace (please wait)...\n");
#endif

    signal(SIGALRM, (void *) catch);
    alarm(MAXWAIT);

    /* open a pipe.  pipefd[0] is read end, pipefd[1] is write side */
    pipe(pipefd);
    /* spawn a process to perform the trace */
    switch (pid=fork()) {
        case -1:
            return(-1);
        case 0: //child
            dup2(pipefd[1], 1); // file descrip 1 is stdout
            execl(tracecmd, tracecmd, "-n", dest, (char *) NULL);
            exit(0);
        default: //parent 
            /* set select timeout and file descriptor of interest */
            FD_ZERO(&fds);
            FD_SET(pipefd[0], &fds);
            timeout.tv_usec = 0;
            memset(result, '\0', BUFSIZE); 

            /* while there is data to read off the pipe */
            while (TRUE) {
                timeout.tv_sec = TRWAIT;
                if ( select(pipefd[0] + 1, &fds, (fd_set *) NULL, 
                     (fd_set *) NULL, &timeout) <= 0 ) {
                    break;
                }
                /* read a byte */
                if (Readn(pipefd[0],&(result[bytes_read]),1)) 
                    bytes_read++;
                /* ensure we haven't overrun our output buffer */
                if (bytes_read >= BIGBUF)
                    return (bytes_read);
            }
    }
    return(bytes_read);
}

#if 0
int main() {
    char tracebuf[BIGBUF];

    runtrace("192.168.1.100", tracebuf);
    runtrace("128.61.2.1", tracebuf);
    printf("string: %s\n", tracebuf);
    exit(0);
}
#endif

#endif
