/****************************************************************************
   Program:     $Id: util.c,v 1.12 2009/01/14 15:19:57 rbeverly Exp $
   Author:      Rob Beverly <rbeverly at csail.mit.edu>
   Date:        $Date: 2009/01/14 15:19:57 $
   Description: Spoofer utility routines
****************************************************************************/
#include "spoof.h"
#include <stdarg.h>

/**
 * Fatal error condition function 
 */
void fatal(char *fmt, ...) {
    va_list ap;
    fflush(stdout);
    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    exit(-1);
}

/**
 * Fatal error condition function; prints error
 */
void pfatal(char *err) {
    fflush(stdout);
    perror(err);
    exit(-1);
}

/**
 * binDump -  Debug routine for examining raw packets/buffer/char strings 
 *
 * @param[in]   msg     pointer to message buffer
 * @param[in]   len     length of buffer to print
 */
void binDump(unsigned char *msg, int len) {
    int i;
    for (i=1;i<=len;i++) {
        if (len > 16) {
            printf("%02x ", *msg++);
            if (i%4 == 0) printf("\n");
        } else {
            printf("%02x", *msg++);
        }
    }
    printf("\n");
}

/**
 * byte32Swap - A ntohl() for big endian architectures who null macro it 
 */
unsigned long byte32Swap(unsigned long in) {
    return ( (in & 0x000000ff)<<24 | (in & 0x0000ff00)<<8 |
             (in & 0x00ff0000)>>8 | (in & 0xff000000)>>24 );
}

/**
 * byte16Swap - A ntoh() for big endian architectures who null macro it 
 */
unsigned short byte16Swap(unsigned short in) {
    return ( (in & 0x00ff)<<8 | (in & 0xff00)>>8 );
}

/**
 * checkroot -  Check for root priviledges 
 */
void checkroot() {
#ifndef _WIN32
    if (getuid() != 0) {
        fatal("** Must be run as root!\n");
    }
#endif
}

/**
 * win_init -  Windows socket API startup 
 */
void win_init() {
#ifdef _WIN32
    WSADATA wsd;
    if (WSAStartup(MAKEWORD(2, 2), &wsd) != 0) {
        winfatal("** WSAStartup() failed");
    }
#endif
}

/**
 * banner - Hi there 
 */
void banner() {
    printf(">> MIT ANA Spoofer Project\n");
    printf(">> IP Source Spoofing Tester v0.%d\n", VERSION);
    printf(">> http://%s\n", REPORT_HOST);
    printf(">> Copyright (c) 2004-2009 Robert Beverly\n");
    printf(">>\n");
}

/**
 * confirm - Ask for user input before proceeding
 */
void confirm() {
    char c;

    fprintf(stdout, "Continue? [y/N]....");
    fflush(stdout);
    fread(&c, sizeof(char), 1, stdin);
    if ( !(c == 'y') || (c == 'Y') ) 
        fatal("Aborting.\n");
}

/**
 * spoofSleep -  Portable microsecond random sleep 
 */
void spoofSleep() {
    unsigned int gap;

    /* random interpacket delay 0-0.5s */
    gap = (unsigned int)((double)rand() / ((double)RAND_MAX + 1) * 500);
#ifdef _WIN32
    Sleep(gap);
#else
    usleep(gap * 1000);
#endif
}

/**
 * timestamp - form time/date string
 */
char *timestamp() {
   static char str[BUFSIZE];
   time_t clock;
   struct tm *t;

   clock = time(NULL);
   t = localtime(&clock);
   snprintf(str, sizeof(str), "%02d/%02d %02d:%02d:%02d", t->tm_mon + 1,
      t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec);
   return (str);
}
